@@ -1,17 +1,43 @@
 """Wine prefix management"""
+# Standard Library
 import os
-from lutris.util.wine.registry import WineRegistry
-from lutris.util.log import logger
+
+# Lutris Modules
 from lutris.util import joypad, system
-from lutris.util.xdgshortcuts import get_xdg_entry
 from lutris.util.display import DISPLAY_MANAGER
+from lutris.util.log import logger
+from lutris.util.wine.registry import WineRegistry
+from lutris.util.xdgshortcuts import get_xdg_entry
 
 DESKTOP_KEYS = ["Desktop", "Personal", "My Music", "My Videos", "My Pictures"]
 DEFAULT_DESKTOP_FOLDERS = ["Desktop", "My Documents", "My Music", "My Videos", "My Pictures"]
 DESKTOP_XDG = ["DESKTOP", "DOCUMENTS", "MUSIC", "VIDEOS", "PICTURES"]
 
 
+def is_prefix(path):
+    """Return True if the path is prefix"""
+    return os.path.isdir(os.path.join(path, "drive_c")) \
+        and os.path.exists(os.path.join(path, "user.reg"))
+
+
+def find_prefix(path):
+    """Given an executable path, try to find a Wine prefix associated with it."""
+    dir_path = path
+    if not dir_path:
+        logger.info("No path given, unable to guess prefix location")
+        return
+    while dir_path != "/" and dir_path:
+        dir_path = os.path.dirname(dir_path)
+        if is_prefix(dir_path):
+            return dir_path
+        for prefix_dir in ("prefix", "pfx"):
+            prefix_path = os.path.join(dir_path, prefix_dir)
+            if is_prefix(prefix_path):
+                return prefix_path
+
+
 class WinePrefixManager:
+
     """Class to allow modification of Wine prefixes without the use of Wine"""
 
     hkcu_prefix = "HKEY_CURRENT_USER"
@@ -27,9 +53,7 @@ class WinePrefixManager:
         try:
             self.desktop_integration()
         except OSError as ex:
-            logger.error(
-                "Failed to setup desktop integration, the prefix may not be valid."
-            )
+            logger.error("Failed to setup desktop integration, the prefix may not be valid.")
             logger.exception(ex)
 
     def get_registry_path(self, key):
@@ -44,9 +68,7 @@ class WinePrefixManager:
     def get_key_path(self, key):
         if key.startswith(self.hkcu_prefix):
             return key[len(self.hkcu_prefix) + 1:]
-        raise ValueError(
-            "The key {} is currently not supported by WinePrefixManager".format(key)
-        )
+        raise ValueError("The key {} is currently not supported by WinePrefixManager".format(key))
 
     def get_registry_key(self, key, subkey):
         registry = WineRegistry(self.get_registry_path(key))
@@ -81,8 +103,7 @@ class WinePrefixManager:
         desktop_folders = []
         for key in DESKTOP_KEYS:
             folder = self.get_registry_key(
-                self.hkcu_prefix
-                + "/Software/Microsoft/Windows/CurrentVersion/Explorer/Shell Folders",
+                self.hkcu_prefix + "/Software/Microsoft/Windows/CurrentVersion/Explorer/Shell Folders",
                 key,
             )
             if not folder:
@@ -91,9 +112,13 @@ class WinePrefixManager:
             desktop_folders.append(folder[folder.rfind("\\") + 1:])
         return desktop_folders or DEFAULT_DESKTOP_FOLDERS
 
-    def desktop_integration(self, desktop_dir=None, restore=False):
+    def desktop_integration(self, desktop_dir=None, restore=False):  # noqa: C901
         """Overwrite desktop integration"""
+        # pylint: disable=too-many-branches
+        # TODO: reduce complexity (18)
         user = os.getenv("USER")
+        if not user:
+            user = 'lutrisuser'
         user_dir = os.path.join(self.path, "drive_c/users/", user)
         desktop_folders = self.get_desktop_folders()
 
@@ -126,10 +151,7 @@ class WinePrefixManager:
                 if restore and not os.path.isdir(path):
                     src_path = get_xdg_entry(DESKTOP_XDG[i])
                     if not src_path:
-                        logger.error(
-                            "No XDG entry found for %s, launcher not created",
-                            DESKTOP_XDG[i]
-                        )
+                        logger.error("No XDG entry found for %s, launcher not created", DESKTOP_XDG[i])
                     else:
                         os.symlink(src_path, path)
                     # We don't need all the others process of the loop
@@ -142,8 +164,7 @@ class WinePrefixManager:
                         # There is supposedly a None value in there
                         # The current code shouldn't allow that
                         # Just raise a exception with the values
-                        raise RuntimeError("Missing value desktop_dir=%s or item=%s"
-                                           % (desktop_dir, item))
+                        raise RuntimeError("Missing value desktop_dir=%s or item=%s" % (desktop_dir, item))
 
                     os.makedirs(src_path, exist_ok=True)
                     os.symlink(src_path, path)
